

class ErrorNotification:

    def __init__(self):
        self.errors = []

    def notify_error(self,error):
        self.errors.append(error)

    def collect_notifications(self, notification):
        for error in notification.iterate_errors():
            self.notify_error(error)

    def iterate_errors(self):
        for error in self.errors:
            yield error

    def has_errors(self):
        return len(self.errors) > 0





NUEVO = "nuevo"
HISTORICO = "histórico"

class Collection:

    def __init__(self, document, raw_code, customer, date, amount, errors):
        collection = dict()

        # xubio transaction
        self.document = document
        self.raw_code = raw_code or ""

        # customer
        self.customer = customer

        # transaction details
        self.date = date
        self.amount = amount

        # defaults
        self.version = HISTORICO
        self.sales_order = ""
        self.payments = ""

        if "-" in raw_code:
            self.version = NUEVO
            order_and_payments = raw_code.split("-")
            self.sales_order, *self.payments = order_and_payments

        if self.sales_order and len(self.sales_order) != 5:
            error = "Cobranza con orden de compra errónea (%s). Documento: %s" % (self.sales_order, self.document)
            errors.append(error)

    def to_dict(self):
        collection = {}

        collection['version'] = self.version
        collection['date'] = self.date
        collection['customer'] = self.customer
        collection['amount'] = self.amount
        collection['sales_order'] = self.sales_order
        collection['payments'] = self.payments

        return collection



class ErrorsIterator:

    def __init__(self):
        self.errors = []

    def add_error(self,error):
        self.errors.append(error)

    def iterate_errors(self):
        for error in self.errors:
            yield error





class BillExtractorResults:

    def __init__(self):
        self.bills = {}
        self.errors = []
        pass

    def append_bill(self,bill):
        self.bills[bill.get_document_type_and_number()] = bill.get_amount()

    def get_errors(self):
        return self.errors

    def get_bills(self):
        return self.bills




def parse_debits_lines(page_data):
    last_row = page_data.max_row + 1

    debit_lines = []

    for line_num in range(8, last_row):
        line_mapping = map_line(line_num, page_data)
        debit_lines.append(DebitLine(line_mapping))

    for line in debit_lines:
        # print(line.cbu_bloque_1, line.cbu_bloque_2, line.monto)
        pass

    return debit_lines


def map_line(row, page_data):
    raw_amount = page_data.cell(row=row, column=5).value
    amount = None

    if raw_amount:
        amount = format_amount(raw_amount)

    return {
        'sheet_name': page_data.title,
        'row': row,
        'tipo_novedad': page_data.cell(row=row, column=1).value,
        'cbu_bloque_1':  page_data.cell(row=row, column=2).value,
        'cbu_bloque_2': page_data.cell(row=row, column=3).value,
        'id_cliente': page_data.cell(row=row, column=4).value,
        'monto': amount,
        'monto_numero': raw_amount
    }


def format_amount(amount):
    return str(round(amount * 100)).zfill(10)


class DebitLine:

    def __init__(self, line_mapping):
        self._sheet_name = line_mapping['sheet_name']
        self._row = line_mapping['row']
        self._tipo_novedad = line_mapping['tipo_novedad']
        self._cbu_bloque_1 = line_mapping['cbu_bloque_1']
        self._cbu_bloque_2 = line_mapping['cbu_bloque_2']
        self._id_cliente = line_mapping['id_cliente']
        self._monto = line_mapping['monto']
        self._monto_numero = line_mapping['monto_numero']


    @property
    def sheet_name(self):
        return self._sheet_name


    @property
    def row(self):
        return self._row


    @property
    def tipo_novedad(self):
        return self._tipo_novedad


    @property
    def cbu_bloque_1(self):
        return self._cbu_bloque_1


    @property
    def cbu_bloque_2(self):
        return self._cbu_bloque_2


    @property
    def id_cliente(self):
        return self._id_cliente


    @property
    def monto(self):
        return self._monto


    @property
    def monto_numero(self):
        return self._monto_numero


def get_advance_payments_columns():
    config_list = list()
    config_list.append(("A", "Documento", 'document'))
    config_list.append(("B", "Customer", 'customer'))
    config_list.append(("C", "Monto total", 'total_purchase_value'))
    config_list.append(("D", "Anticipo calculado", 'advance_payment'))
    return config_list


def null_item_mapper(item, key):
    return item[key]


class BasicBuilder:

    def __init__(self, data_list, columns_configuration=None):
        self.data_list = data_list
        self.headers = []
        self.column_mappings = {}
        self.header_row = 1
        self.first_data_row = 2

        self.manual_values = {}

        if columns_configuration:
            self.configure_columns(columns_configuration)

    def set_header_row(self, header_row):
        self.header_row = header_row

    def set_first_data_row(self, data_row):
        self.first_data_row = data_row

    def add_header(self, column_letter, title):
        self.headers.append({"column_letter": column_letter, "title": title})

    def map_column(self, column_letter, item_key, mapper=null_item_mapper):
        self.column_mappings[column_letter] = {'key': item_key, 'mapper': mapper}

    def configure_column(self, column_letter, title, item_key):
        self.add_header(column_letter, title)
        self.map_column(column_letter, item_key)

    def configure_columns(self, list_of_columns):
        for column_configuration in list_of_columns:
            self.configure_column(*column_configuration)

    def set_manual_cell(self, cell_position, cell_value):
        self.manual_values[cell_position] = cell_value

    def build_sheet_data(self):
        sheet_data = {}

        sheet_data.update(self.__build_headers())
        sheet_data.update(self.__build_data_rows())
        sheet_data.update(self.manual_values)

        return sheet_data

    def __build_headers(self):
        headers_map = {}

        for header in self.headers:
            cell = self.__get_cell_location(header['column_letter'], self.header_row)
            headers_map[cell] = header['title']

        return headers_map

    def __build_data_rows(self):
        rows_map = {}

        for index, item in enumerate(self.data_list):
            row_num = str(index + self.first_data_row)

            for column_letter, item_key_and_mapper in self.column_mappings.items():
                cell = self.__get_cell_location(column_letter, row_num)

                item_mapper = item_key_and_mapper['mapper']
                item_key = item_key_and_mapper['key']

                rows_map[cell] = item_mapper(item, item_key)

        return rows_map

    def __get_cell_location(self, header, row):
        return ("%s%s" % (header, row))



from credisur.models import AccountReceivable

def account_receivable_extractor(results, unpacker):

    document_date = unpacker.get_value_at(1)
    due_date = unpacker.get_value_at(2)
    document = unpacker.get_value_at(3)
    customer = unpacker.get_value_at(4)
    raw_code = unpacker.get_value_at(9)

    customers = results.get_customers()
    bills = results.get_bills()

    calendar_ops = results.get_calendar_ops()
    last_date_of_month = results.get_last_date_of_month()
    first_day_of_current_month = results.get_first_day_of_current_month()

    collections = results.get_collections()
    collections_for_customers = results.get_collections_for_customers()

    list_of_advance_payments = results.get_list_of_advance_payments()

    customers_without_payments_due = results.get_customers_without_payments_due()
    customers_in_last_payment = results.get_customers_in_last_payment()

    errors = results.get_errors()

    if "Cobranza" in document: return
    if not raw_code: return

    if len(raw_code.split("-")) < 4: return

    line_amount = float(unpacker.get_value_at(8))
    line_balance = unpacker.get_value_at(8)

    customer_data = customers[customer]

    account_receivable = AccountReceivable(document_date, due_date, document,
                                           customer, raw_code, customer_data, line_amount,
                                           line_balance, calendar_ops)

    if not account_receivable.is_historic_and_due_for(last_date_of_month):
        return

    if not account_receivable.validate_payment_plan(errors):
        return

    account_receivable.configure_previous_collections(collections, collections_for_customers)
    account_receivable.compute_last_collection_date()

    account_receivable.compute_due_payment(bills, first_day_of_current_month, last_date_of_month)

    account_receivable.validate_low_advance_payment(100, errors)
    account_receivable.validate_advance_payment(errors)

    account_receivable.add_to_list_if_advance_payments(list_of_advance_payments)

    if not account_receivable.validate_total_sale_amount_and_plan_value(errors):
        return

    if not account_receivable.has_due_payments(customers_without_payments_due):
        return

    if not account_receivable.validate_person(errors):
        return

    account_receivable.add_to_list_if_in_last_payment(customers_in_last_payment, first_day_of_current_month)

    results.append_account_to_collect(account_receivable)


def to_google_num(value):
    return "{:}".format(value).replace(".", ",")


def unpack_dates(date_value):
    date = date_value.strftime("%d/%m/%Y") if date_value else "N/D"
    week = int(date_value.strftime("%U")) + 1 if date_value else "N/D"
    year = date_value.strftime("%Y") if date_value else "N/D"
    return date, week, year



import os
import credisur.excelbuilder as excelbuilder
from credisur.exceladapter import ExcelWriter
import time

def parse_bank_files(cwd):
    print("parse_bank_files")
    results = list()

    if not os.path.isdir(cwd + '/outputs/'):
        missing_path = ("/".join(cwd.split("/")) + '/outputs/')
        print ("ERROR: El directorio %s no existe" % (missing_path,))
        return

    for filepath in files("inputs/"):
        results.extend(list(parse_file(filepath)))

    filepath = cwd + '/outputs/' + 'rendicion' + time.strftime("%Y%m%d-%H%M%S") + '.xlsx'
    writer = configure_writer(filepath)
    build_excel(writer, results, bank_columns_config())
    writer.save()


def files(path):
    for file in os.listdir(path):
        if not file.endswith("rendi"):
            continue

        filepath = os.path.join(path, file)

        if not os.path.isfile(filepath):
            continue

        yield filepath

def parse_file(filepath):
    with open(filepath, "r") as file:
        for line in file:
            parsed_line = parse_line(line)
            if not parsed_line:
                continue
            yield parsed_line

def parse_line(line):
    tipo_de_novedad = line[0:1]
    cuit_originante = line[1:11]
    sector = line[12:14]
    prestacion = line[15:24]
    fecha_vencimiento = line[25:33]
    cbu_bloque_1 = line[33:41]
    filler_cbu_2 = line[41:43]
    cbu_bloque_2 = line[44:58]
    id_cliente = line[58:79]
    vto_debito_original = line[80:87]
    ref_debito = line[88:102]
    importe = int(line[103:112].lstrip("0"))
    moneda_debito = line[113:114]
    fecha_segundo_vencimiento= line[115:122]
    importe_segundo_vencimiento = line[123:132]
    fecha_tercer_vencimiento = line[133:140]
    importe_tercer_vencimiento = line[141:150]
    identificador_pagador_nuevo = line[151:172]
    codigo_rechazos = line[173:176]

    if not tipo_de_novedad == "D":
        print(tipo_de_novedad)
        return

    return {
        'tipo_de_novedad': tipo_de_novedad,
        'cuit_originante': cuit_originante,
        'sector': sector,
        'prestacion': prestacion,
        'fecha_vencimiento': fecha_vencimiento,
        'cbu_bloque_1': cbu_bloque_1,
        'filler_cbu_2': filler_cbu_2,
        'cbu_bloque_2': cbu_bloque_2,
        'id_cliente': id_cliente,
        'vto_debito_original': vto_debito_original,
        'ref_debito': ref_debito,
        'importe': importe,
        'moneda_debito': moneda_debito,
        'fecha_segundo_vencimiento': fecha_segundo_vencimiento,
        'importe_segundo_vencimiento': importe_segundo_vencimiento,
        'fecha_tercer_vencimiento': fecha_tercer_vencimiento,
        'importe_tercer_vencimiento': importe_tercer_vencimiento,
        'identificador_pagador_nuevo': identificador_pagador_nuevo,
        'codigo_rechazos': codigo_rechazos
    }

def bank_columns_config():
    # D E F H I L S

    config_list = list()
    config_list.append(("A", "Prestación", 'prestacion'))
    config_list.append(("B", "Fecha Vencimiento", 'fecha_vencimiento'))
    config_list.append(("C", "CBU Bloque 1", 'cbu_bloque_1'))
    config_list.append(("D", "CBU Bloque 2", 'cbu_bloque_2'))
    config_list.append(("E", "ID Cliente", 'id_cliente'))
    config_list.append(("F", "Importe", 'importe'))
    config_list.append(("G", "Código Rechazos", 'codigo_rechazos'))

    return config_list

def configure_writer(filepath):
    return ExcelWriter(filepath)

def build_excel(excelwriter, transactions, columns_config):
    builder = excelbuilder.BasicBuilder(transactions, columns_config)
    excelwriter.build_sheet("Resultados Débito", builder.build_sheet_data())

def print_not_empty(label, str):
    if str.strip():
        print(label, str)


import os
from .errorsiterator import ErrorsIterator

def validate_file(excel_path):
    errors = ErrorsIterator()
    if not os.path.isfile(excel_path):
        errors.add_error("El archivo %s no existe." % (excel_path))

    return errors
